from django.db import models
from .auth_models import User

class Genre(models.Model):
    name = models.CharField(max_length=100)
    users = models.ManyToManyField(User, related_name="genres")

    def __str__(self):
        return f"{self.name}"

class Theme(models.Model):
    name = models.CharField(max_length=100)
    genres = models.ManyToManyField(Genre, related_name="themes")

    def __str__(self):
        return f"{self.name}"


class Text(models.Model):
    title = models.CharField(max_length=100, null=True)
    content = models.TextField(null=True)
    creation_date = models.DateTimeField()
    hash = models.CharField(max_length=64)

    author = models.ForeignKey(User, on_delete=models.CASCADE)
    genre = models.ForeignKey(Genre, on_delete=models.SET_NULL, null=True)
    theme = models.ForeignKey(Theme, on_delete=models.SET_NULL, null=True)

    is_published = models.BooleanField(default=False)

    def __str__(self):
        return f"{self.title} by {self.author}"


class Review(models.Model):
    author = models.ForeignKey(User, on_delete=models.CASCADE)
    text = models.ForeignKey(Text, on_delete=models.CASCADE)

    creation_date = models.DateTimeField()
    summary = models.TextField(null=True)
    mark = models.IntegerField(null=True)

    was_seen = models.BooleanField(default=False)
    is_published = models.BooleanField(default=False)

    def __str__(self):
        return f"{self.author}'s review of \'{self.text}\'"

class ReviewSegment(models.Model):
    review = models.ForeignKey(Review, on_delete=models.CASCADE)

    position = models.IntegerField()
    length = models.IntegerField()
    comment = models.TextField()

    def __str__(self):
        return f"Segment of {self.review}"
